#include "ac.h"
#include "ac_url.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>


int ac_url_islocal(const char *url) {
	return (url[0] == '/' || !strncmp("file://", url, 7) || !(strchr(url, ':')));
}


const char *ac_url_makelocal(const char *url) {
	struct ac_url *tmp;
	char *b;
	if (url[0] == '/') {
		return url;
	}
	if (!strchr(url, ':')) {
		/* No protocol part specified */
		return url;
	}
	tmp = ac_url_parse(url, NULL);
	if (tmp == NULL) {
		return NULL;
	} else {
		b = tmp->path;
		free(tmp);
		return b;
	}
}

/* Taken from the libcurl source code. */
struct ac_url *ac_url_parse(const char *url, struct ac_url *to)
{
  char *at;
  char *tmp;

  char *path;
  size_t len = strlen(url);

  if (!to) {
	to = malloc(sizeof(struct ac_url));
  }
  to->path = malloc(sizeof(char) * len);
  path = to->path;
  
  /*************************************************************
   * Parse the URL.
   *
   * We need to parse the url even when using the proxy, because we will need
   * the hostname and port in case we are trying to SSL connect through the
   * proxy -- and we don't know if we will need to use SSL until we parse the
   * url ...
   ************************************************************/
  if((2 == sscanf(url, "%15[^:]:%[^\n]",
                  to->scheme,
                  to->path)) && !strcmp(to->scheme, "file")) {
    if(path[0] == '/' && path[1] == '/') {
      /* Allow omitted hostname (e.g. file:/<path>).  This is not strictly
       * speaking a valid file: URL by RFC 1738, but treating file:/<path> as
       * file://localhost/<path> is similar to how other schemes treat missing
       * hostnames.  See RFC 1808. */

      /* This cannot be done with strcpy() in a portable manner, since the
         memory areas overlap! */
      memmove(path, path + 2, strlen(path + 2)+1);
    }
    /*
     * we deal with file://<host>/<path> differently since it supports no
     * hostname other than "localhost" and "127.0.0.1", which is unique among
     * the URL protocols specified in RFC 1738
     */
    if(path[0] != '/') {
      /* the URL included a host name, we ignore host names in file:// URLs
         as the standards don't define what to do with them */
      char *ptr=strchr(path, '/');
      if(ptr) {
        /* there was a slash present

           RFC1738 (section 3.1, page 5) says:

           The rest of the locator consists of data specific to the scheme,
           and is known as the "url-path". It supplies the details of how the
           specified resource can be accessed. Note that the "/" between the
           host (or port) and the url-path is NOT part of the url-path.

           As most agents use file://localhost/foo to get '/foo' although the
           slash preceding foo is a separator and not a slash for the path,
           a URL as file://localhost//foo must be valid as well, to refer to
           the same file with an absolute path.
        */

        if(ptr[1] && ('/' == ptr[1]))
          /* if there was two slashes, we skip the first one as that is then
             used truly as a separator */
          ptr++;

        /* This cannot be made with strcpy, as the memory chunks overlap! */
        memmove(path, ptr, strlen(ptr)+1);
      }
    }
    strcpy(to->scheme, "file"); /* store protocol string lowercase */
  }
  else {
    /* clear path */
    path[0]=0;

    if (2 > sscanf(url,
                   "%15[^\n:]://%[^\n/]%[^\n]",
                   to->scheme,
                   to->hostname, path)) {

      /*
       * The URL was badly formatted, let's try the browser-style _without_
       * protocol specified like 'http://'.
       */
      if((1 > sscanf(url, "%[^\n/]%[^\n]",
                     to->hostname, path)) ) {
        /*
         * We couldn't even get this format.
         */
        AC_DEBUG("<url> malformed");
        return NULL;
      }

      /*
	   * Since there is no protocol part specified we add file:// as protocol
	   */
	  strcpy(to->scheme, "file");

    }
  }

  /* We search for '?' in the host name (but only on the right side of a
   * @-letter to allow ?-letters in username and password) to handle things
   * like http://example.com?param= (notice the missing '/').
   */
  at = strchr(to->hostname, '@');
  if(at)
    tmp = strchr(at+1, '?');
  else
    tmp = strchr(to->hostname, '?');

  if(tmp) {
    /* We must insert a slash before the '?'-letter in the URL. If the URL had
       a slash after the '?', that is where the path currently begins and the
       '?string' is still part of the host name.

       We must move the trailing part from the host name and put it first in
       the path. And have it all prefixed with a slash.
    */

    size_t hostlen = strlen(tmp);
    size_t pathlen = strlen(path);

    /* move the existing path plus the zero byte forward, to make room for
       the host-name part */
    memmove(path+hostlen+1, path, pathlen+1);

     /* now copy the trailing host part in front of the existing path */
    memcpy(path+1, tmp, hostlen);

    path[0]='/'; /* prepend the missing slash */

    *tmp=0; /* now cut off the hostname at the ? */
  }
  else if(!path[0]) {
    /* if there's no path set, use a single slash */
    strcpy(path, "/");
  }

  /* If the URL is malformatted (missing a '/' after hostname before path) we
   * insert a slash here. The only letter except '/' we accept to start a path
   * is '?'.
   */
  if(path[0] == '?') {
    /* We need this function to deal with overlapping memory areas. We know
       that the memory area 'path' points to is 'urllen' bytes big and that
       is bigger than the path. Use +1 to move the zero byte too. */
    memmove(&path[1], path, strlen(path)+1);
    path[0] = '/';
  }

  /*
   * So if the URL was A://B/C,
   *   conn->protostr is A
   *   conn->host.name is B
   *   data->reqdata.path is /C
   */
  return to;
}

/**
 * Frees the _members_ of the given url structure, NOT THE URL ITSELF!
 * @param url: The url structure to free
 */
void ac_url_free(struct ac_url *url) {
	free(url->path);
}
