#ifndef _AC_URL_H
#define _AC_URL_H

#define AC_URL_SCHEMELEN 16
#define AC_URL_MAXHOSTNAMELEN 256

#define AC_URL_MALFORMED 1

/**
 * The structure used to represent a url
 */
struct ac_url {
	/**
	 * The scheme part (protocol specifier)
	 */
	char scheme[AC_URL_SCHEMELEN+1];
	/**
	 * The name of the host
	 */
	char hostname[AC_URL_MAXHOSTNAMELEN+1];
	/**
	 * The path
	 */
	char *path;
};

/**
 * Checks if the given url is a local resource
 * @param url The url to check
 * @return 0 If the url is non-local, a non-zero value otherwise.
 */
int ac_url_islocal(const char *url);

/**
 * Creates a local path of the given url. If no protocol part is specified
 * url is return, otherwise the path of the passed URL is returned. If no
 * valid URL is given, NULL is returned.
 * @param url The url to make local
 * @return A local form of the url
 */
const char *ac_url_makelocal(const char *url);

/**
 * Parse URL and fill in the ac_url structure.
 * @param url The url to parse
 * @param to The struct ac_url structure to fill out. If null a new
 *   one is allocated.
 */
struct ac_url *ac_url_parse(const char *url, struct ac_url *to);

/**
 * Frees the _members_ of the given url structure, NOT THE URL ITSELF!
 * @param url: The url structure to free
 */
void ac_url_free(struct ac_url *url);

#endif
